//
//  ViewController.swift
//  excavator
//
//  Created by kunii on 2018/09/02.
//  Copyright © 2018年 Takahiro Kunii. All rights reserved.
//

import UIKit
import ARKit

class ViewController: UIViewController {

    // ARシーンを表示する
    var sceneView:ARSCNView!
    
    //  ロボットアームのルートノード
    var excavator:SCNNode!
    
    override func viewDidLoad() {
        super.viewDidLoad()

        // ARシーンを表示するビューの作成とself.view小画面としての登録
        sceneView = ARSCNView(frame: self.view.bounds)
        sceneView.autoresizingMask = [.flexibleWidth, .flexibleHeight]
        self.view.addSubview(sceneView)

        // 3D文字列を管理する仮想シーン環境
        let scene = SCNScene()
        
        // ARシーンを表示するビューへの仮想シーン環境の登録
        sceneView.scene = scene
        
        //  光源ノードを作り3D仮想空間のルートノードに追加
        let lightNode = SCNNode()
        let light = SCNLight()
        lightNode.light = light
        scene.rootNode.addChildNode(lightNode)
        
        //  環境光の追加
        let ambientLightNode = SCNNode()
        let ambientLight = SCNLight()
        ambientLight.type = .ambient
        ambientLight.color = UIColor(white: 0.5, alpha: 1)
        ambientLightNode.light = ambientLight

        //  ロボットアーム
        excavator = createExcavator()
        excavator.position = SCNVector3(x:0, y:-1, z:-3)
        scene.rootNode.addChildNode(excavator)
        
        //  ドラッグ検出用に登録
        let panGestureRecognizer = UIPanGestureRecognizer(target: self, action: #selector(moveBoom(_:)))
        self.view.addGestureRecognizer(panGestureRecognizer)
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }

    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        
        // ARセッションの初期設定
        let configuration = ARWorldTrackingConfiguration()
        
        // ARセッション開始
        sceneView.session.run(configuration)
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        super.viewWillDisappear(animated)
        
        // セッション停止
        sceneView.session.pause()
    }

    //  ロボットアームのノード作成
    func createExcavator() -> SCNNode {
        //  土台
        let base = SCNBox()
        let baseNode = SCNNode(geometry: base)
        
        //  肩
        let shoulder = SCNSphere()
        let shoulderNode = SCNNode(geometry: shoulder)
        shoulderNode.name = "shoulder"  //  baseNodeから検索できるように名前をつける
        baseNode.addChildNode(shoulderNode)
        
        //  ブーム
        let boom = SCNCone(topRadius: 0.3, bottomRadius: 0.4, height: 3)
        let boomNode = SCNNode(geometry: boom)
        boomNode.name = "boom"  //  baseNodeから検索できるように名前をつける
        shoulderNode.addChildNode(boomNode)
        
        //  アーム
        let arm = SCNCone(topRadius: 0.2, bottomRadius: 0.3, height: 2)
        let armNode = SCNNode(geometry: arm)
        armNode.name = "arm"  //  baseNodeから検索できるように名前をつける
        boomNode.addChildNode(armNode)
        
        //  バケット
        let bucket = SCNPyramid()
        let bucketNode = SCNNode(geometry: bucket)
        bucketNode.name = "bucket"  //  baseNodeから検索できるように名前をつける
        armNode.addChildNode(bucketNode)
        
        //  土台はpositionの示す位置が底辺になるようにする
        baseNode.pivot = SCNMatrix4MakeTranslation(0, baseNode.geometry!.boundingBox.min.y, 0)
        
        //  肩である球体は土台の最上部が、球の中心位置になるように配置
        shoulderNode.position = SCNVector3(x:0, y:baseNode.geometry!.boundingBox.max.y, z:0)
        
        //  ブームはpositionの示す位置が底辺になるようにし、肩の最上部 - 30cm に配置（ちょっとめり込ます）
        boomNode.pivot = SCNMatrix4MakeTranslation(0, boomNode.geometry!.boundingBox.min.y, 0)
        boomNode.position = SCNVector3(x:0, y:shoulderNode.geometry!.boundingBox.max.y - 0.3, z:0)
        
        //  アームはpositionの示す位置が底辺になるようにし、ブームの最上部に配置
        armNode.pivot = SCNMatrix4MakeTranslation(0, armNode.geometry!.boundingBox.min.y, 0)
        armNode.position = SCNVector3(x:0, y:boomNode.geometry!.boundingBox.max.y, z:0)
        
        //  バケットはアームの最上部に配置
        bucketNode.position = SCNVector3(x:0, y:armNode.geometry!.boundingBox.max.y, z:0)
        
        //  色分け
        base.firstMaterial?.diffuse.contents = UIColor.red
        shoulder.firstMaterial?.diffuse.contents = UIColor.yellow
        boom.firstMaterial?.diffuse.contents = UIColor.blue
        arm.firstMaterial?.diffuse.contents = UIColor.green
        bucket.firstMaterial?.diffuse.contents = UIColor.orange
        return baseNode
    }
    
    /// ドラッグ中の指の存在領域を示す　-1:負回転　0:停止　1:正回転
    private var directionRegion = 0
    
    /// 前回moveBoomが呼び出された時のdirectionRegionの値を記録
    private var lastDirectionRegion = 0
    
    /// ドラッグを検出した
    @objc func moveBoom(_ panRecognizer:UIPanGestureRecognizer) {
        if panRecognizer.state == .changed {
            
            let translation = panRecognizer.translation(in: panRecognizer.view)
            
            //  ドラッグ中の指の存在領域の決定　停止領域の上下20ポイント分は
            //  無効な領域になっていてdirectionRegionを変化させない
            if translation.y < -50 {
                directionRegion = 1
            } else if translation.y > 50 {
                directionRegion = -1
            } else if translation.y > -30 && translation.y < 30 {
                directionRegion = 0
            }
            
            if lastDirectionRegion != directionRegion {
                //  領域が変わった
                lastDirectionRegion = directionRegion
                
                if let node = excavator.childNode(withName: "boom", recursively: true) {
                    //  まず停止：画面上の角度を設定することでアニメーション中なら強制停止になる
                    node.eulerAngles.z = node.presentation.eulerAngles.z
                    if directionRegion != 0 {
                        //  停止領域でないなら領域方向に一定速度でアニメーションする
                        SCNTransaction.begin()
                        SCNTransaction.animationDuration = 5
                        node.eulerAngles.z += 1.0 * Float(directionRegion)
                        SCNTransaction.commit()
                    }
                }
            }
        }
        
        //  ドラッグ終了もしくはキャンセル時はアニメーションを強制的に停止
        if panRecognizer.state == .ended || panRecognizer.state == .cancelled {
            if let node = excavator.childNode(withName: "boom", recursively: true) {
                node.eulerAngles.z = node.presentation.eulerAngles.z
            }
        }
    }
}

